@@ -186,12 +186,11 @@ gst_flv_mux_class_init (GstFlvMuxClass * klass)
       GST_DEBUG_FUNCPTR (gst_flv_mux_request_new_pad);
   gstelement_class->release_pad = GST_DEBUG_FUNCPTR (gst_flv_mux_release_pad);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&videosink_templ));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&audiosink_templ));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &videosink_templ);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &audiosink_templ);
+  gst_element_class_add_static_pad_template (gstelement_class, &src_templ);
   gst_element_class_set_static_metadata (gstelement_class, "FLV muxer",
       "Codec/Muxer",
       "Muxes video/audio streams into a FLV stream",
@@ -1337,18 +1336,18 @@ gst_flv_mux_write_header (GstFlvMux * mux)
   return GST_FLOW_OK;
 
 failure_header:
-    gst_buffer_unref (metadata);
+  gst_buffer_unref (metadata);
 
 failure_metadata:
-    if (video_codec_data != NULL)
-      gst_buffer_unref (video_codec_data);
+  if (video_codec_data != NULL)
+    gst_buffer_unref (video_codec_data);
 
 failure_video_codec_data:
-    if (audio_codec_data != NULL)
-      gst_buffer_unref (audio_codec_data);
+  if (audio_codec_data != NULL)
+    gst_buffer_unref (audio_codec_data);
 
 failure_audio_codec_data:
-    return ret;
+  return ret;
 }
 
 static void
@@ -1599,12 +1598,12 @@ gst_flv_mux_handle_buffer (GstCollectPads * pads, GstCollectData * cdata,
 
     ret = gst_flv_mux_write_header (mux);
     if (ret != GST_FLOW_OK) {
-       gst_buffer_unref (buffer);
-       return ret;
+      gst_buffer_unref (buffer);
+      return ret;
     }
     mux->state = GST_FLV_MUX_STATE_DATA;
 
-    if (GST_COLLECT_PADS_DTS_IS_VALID (cdata))
+    if (cdata && GST_COLLECT_PADS_DTS_IS_VALID (cdata))
       mux->first_timestamp = GST_COLLECT_PADS_DTS (cdata);
     else
       mux->first_timestamp = 0;
